@@ -13,14 +13,18 @@
 #' the codes \code{"rowFactor"} and \code{"colFactor"} are unchanged. 
 #' An empty string is recoded to  \code{"rowFactor"}.
 #' 
-#' 
+#' A special possibility is to include character vector(s) as unnamed list element(s) of `hierarchies`. 
+#' Then the elements of the character vector(s) must be variable names within data. 
+#' This will cause hierarchies to be created from selected data columns by running \code{\link{FindDimLists}}. 
+#' Total coded can be specified by parameter `total` or by naming the character vector. See examples. 
 #'
 #' @param hierarchies List of hierarchies
 #' @param data Matrix or data frame with data containing codes of relevant variables
-#' @param total Within \code{AutoHierarchies}: Vector of total codes (possibly recycled) used when running \code{\link{Hrc2DimList}}.  
+#' @param total Within \code{AutoHierarchies}: Vector of total codes (possibly recycled) used when running \code{\link{Hrc2DimList}} or \code{\link{FindDimLists}}.  
 #' @param hierarchyVarNames Variable names in the hierarchy tables as in \code{\link{HierarchyFix}}
 #' @param combineHierarchies Whether to combine several hierarchies for same variable into a single hierarchy
 #' @param unionComplement Logical vector as in \code{\link{Hierarchies2ModelMatrix}}. The parameter is only in use when hierarchies are combined. 
+#' @param ... Extra unused parameters
 #' 
 #' @seealso \code{\link{DimList2Hierarchy}}, \code{\link{Hierarchy2Formula}}.
 #'
@@ -48,13 +52,72 @@
 #' identical(h3, h4)
 #' 
 #' FindHierarchies(z[, c("geo", "eu", "age")])
+#' 
+#' 
+#' ### Examples with character vector(s) as unnamed list elements.  
+#' 
+#' # Same output as FindHierarchies above
+#' AutoHierarchies(list(c("geo", "eu", "age")), data = z)
+#' 
+#' # Now combined with a named list element 
+#' AutoHierarchies(list(year = yearHier, c("geo", "eu", "age")), data = z)
+#' 
+#' # Total codes by unnamed list element as named character vector 
+#' AutoHierarchies(list(year = yearHier, c(Europe = "geo", "eu", All = "age")), data = z)
+#' 
+#' # Two types of year input. Total codes by using the parameter `total`. 
+#' AutoHierarchies(list("year", year = yearHier, c("geo", "eu", "age")), data = z, 
+#'                 total = c("allYears", "unused", "Tot"))
+#' 
+#' # Avoid combineHierarchies to see effect of each year input separately 
+#' # (even earlier return possible with `combineHierarchies = NA`)
+#' AutoHierarchies(list("year", year = yearHier, c("geo", "eu", "age")), data = z, 
+#'                 total = c("allYears", "unused", "Tot"), combineHierarchies = FALSE)
+#' 
 AutoHierarchies <- function(hierarchies, data = NULL, total = "Total", 
                             hierarchyVarNames = c(mapsFrom = "mapsFrom", mapsTo = "mapsTo", sign = "sign", level = "level"),
-                            combineHierarchies = TRUE, unionComplement = FALSE) {
+                            combineHierarchies = TRUE, unionComplement = FALSE, ...) {
   total <- rep_len(total, length(hierarchies))
+  
+  if (is.null(names(hierarchies))) {
+    names(hierarchies) <- rep(NA, length(hierarchies))
+  }
+  toFindDimLists <- (names(hierarchies) %in% c(NA, "")) & (sapply(hierarchies, is.character))
+  if (any(toFindDimLists)) {
+    if (is.null(data)) {
+      stop("data input needed")
+    }
+    nHierarchies <- rep(1, length(hierarchies))
+    newDimLists <- vector("list", sum(toFindDimLists))
+    indNewDimLists <- which(toFindDimLists)
+    for (i in seq_along(newDimLists)) {
+      total_i <- total[indNewDimLists[i]]
+      names_i <- names(hierarchies[[indNewDimLists[i]]])
+      if (!is.null(names_i)) {
+        ok_names <- !(names_i %in% c(NA, ""))
+        if (!all(!ok_names)) {
+          total_i <- rep(total_i, length(hierarchies[[indNewDimLists[i]]]))
+          total_i[ok_names] <- names_i[ok_names]
+        }
+      }
+      newDimLists[[i]] <- FindDimLists(data[, hierarchies[[indNewDimLists[i]]], drop = FALSE], total = total_i)
+      nHierarchies[indNewDimLists[i]] <- length(newDimLists[[i]])
+    }
+    indHierarchies <- rep(seq_len(length(hierarchies)), nHierarchies)
+    total <- total[indHierarchies]
+    hierarchies <- hierarchies[indHierarchies]
+    for (i in seq_along(newDimLists)) {
+      hierarchies[indHierarchies == indNewDimLists[i]] <- newDimLists[[i]]
+      names(hierarchies)[indHierarchies == indNewDimLists[i]] <- names(newDimLists[[i]])
+    }
+    if (is.na(combineHierarchies)) {  # Early return hack 
+      return(hierarchies)
+    }
+  }
+  
   namesHierarchies <- names(hierarchies)
-  if (is.null(namesHierarchies)) 
-    stop("hierarchies must be a named list")
+  if (any(names(hierarchies) %in% c(NA, ""))) 
+    stop("Unnamed elements of hierarchies could not be automatically handled (try `combineHierarchies = NA`)")
   for (i in 1:length(hierarchies)) {
     hierarchies[[i]] <- AutoHierarchies1(hierarchies[[i]], data = data, total = total[i], hierarchyVarNames = hierarchyVarNames, varName = namesHierarchies[i])
   }
